@@ -1,7 +1,7 @@
 # Simulation "lab notebook" for collecting results, HDF5 version
 #
-# Copyright (C) 2020 Simon Dobson
-# 
+# Copyright (C) 2020--2021 Simon Dobson
+#
 # This file is part of epyc, experiment management in Python.
 #
 # epyc is free software: you can redistribute it and/or modify
@@ -28,40 +28,41 @@ from requests import get
 from requests.exceptions import MissingSchema
 from tempfile import NamedTemporaryFile
 import sys
+from typing import Any
 if sys.version_info >= (3, 8):
-    from typing import Final, Any
+    from typing import Final
 else:
-    # backwards compatibility with Python 35, Python36, and Python37 
-    from typing import Any
     from typing_extensions import Final
 
 class HDF5LabNotebook(LabNotebook):
-    '''A lab notebook that persists itself to an HDF5 file.
-    `HDF5 <https://www.hdfgroup.org/>`_  is a very common format
-    for sharing large scientific datasets, allowing ``epyc`` to interoperate
-    with a larger toolchain.
+    '''A lab notebook that persists itself to an HDF5 file.  `HDF5
+    <https://www.hdfgroup.org/>`_ is a very common format for sharing
+    large scientific datasets, allowing ``epyc`` to interoperate with
+    a larger toolchain.
 
-    ``epyc`` is built on top of the ``h5py`` `Python binding to HDF5 <https://www.h5py.org/>`_,
-    which handles most of the heavy lifting using a lot of machinery for typing
-    and so on matched with ``numpy``. Note that
-    the limitations of HDF5's types mean that some values may have
-    different types when read than when acquired. (See :ref:`hdf5-type-management`
-    for details.)
+    ``epyc`` is built on top of the ``h5py`` `Python binding to HDF5
+    <https://www.h5py.org/>`_, which handles most of the heavy lifting
+    using a lot of machinery for typing and so on matched with
+    ``numpy``. Note that the limitations of HDF5's types mean that
+    some values may have different types when read than when
+    acquired. (See :ref:`hdf5-type-management` for details.)
 
-    The name of the notebook can be a file or a URL. Only files can be created
-    or updated: if a URL is provided then the notebook will be read and
-    immediately marked as locked. This implies that ``create=True`` won't
-    work in conjunction with URLs.
+    The name of the notebook can be a file or a URL. Only files can be
+    created or updated: if a URL is provided then the notebook will be
+    read and immediately marked as locked. This implies that
+    ``create=True`` won't work in conjunction with URLs.
 
     .. important ::
 
-        Note that because of the design of the ``requests`` library used for
-        handling URLs, using a ``file:``-schema URL will result in an exception
-        being raised. Use filenames for accessing files.
+        Note that because of the design of the ``requests`` library
+        used for handling URLs, using a ``file:``-schema URL will
+        result in an exception being raised. Use filenames for
+        accessing files.
 
     :param name: HDF5 file or URL backing the notebook
     :param create: (optional) if True, erase any existing file (defaults to False)
-    :param description: (optional) free text description of the notebook 
+    :param description: (optional) free text description of the notebook
+
     '''
 
     # Latest file format, defining how a notebook is laid out within the HDF5 file
@@ -79,7 +80,7 @@ class HDF5LabNotebook(LabNotebook):
     VERSION : Final[str] = 'version'                 #: Attribute holding the version of file structure used.
     DESCRIPTION : Final[str] = 'description'         #: Attribute holding the notebook and result set descriptions.
     CURRENT : Final[str] = 'current-resultset'       #: Attribute holding the tag of the current result set.
-    LOCKED : Final[str] = 'locked'                   #: Attribute flagging as result set or notebook as being locked to further changes.
+    LOCKED : Final[str] = 'locked'                   #: Attribute flagging a result set or notebook as being locked to further changes.
 
     def __init__(self, name : str, create : bool =False, description : str =None):
         # create an empty file structure
@@ -100,10 +101,10 @@ class HDF5LabNotebook(LabNotebook):
         # check for file needing creation
         created = (not self._isRemote) and (create or not os.path.isfile(name))
         if created:
-            self._create(name)                
+            self._create(name)
 
         # perform the normal initialisation
-        super(HDF5LabNotebook, self).__init__(name, description)
+        super().__init__(name, description)
 
         if not created:
             # load notebook from file if it wasn't newly created
@@ -163,10 +164,10 @@ class HDF5LabNotebook(LabNotebook):
 
 
     # ---------- Protocol for managing the HDF5 file----------
-    
+
     def _create(self, name : str):
         '''Create the HDF5 file to back this notebook.
-        
+
         :param name: the filename
         :param description: the free text description of this notebook'''
         self._file = h5py.File(name, 'w')
@@ -189,7 +190,7 @@ class HDF5LabNotebook(LabNotebook):
                 # open the file read-only
                 self._file = h5py.File(tf.name, 'r')   # can only be read
             else:
-                # open as a file 
+                # open as a file
                 self._file = h5py.File(self.name(), 'a')
 
     def _close(self):
@@ -202,6 +203,15 @@ class HDF5LabNotebook(LabNotebook):
         if self._isRemote:
             self.finish(commit=False)   # don't try to commit it when finishing
 
+    def _purge(self):
+        '''Delete any HDF5 datasets that relate to deleted result sets.'''
+        tags = self.resultSets()
+        g = self._file
+        for tag in g.keys():
+            if tag not in tags:
+                # result set with this tag has been deleted, remove the corresponding dataset
+                del g[tag]
+
     def _write(self, tag : str):
         '''Write the given result set to the file.
 
@@ -230,7 +240,7 @@ class HDF5LabNotebook(LabNotebook):
             g.attrs.create(k, self._asString(v), dtype=h5py.string_dtype())
 
         # write out the locked flag
-        g.attrs.create(self.LOCKED, rs.isLocked(), dtype=numpy.bool)
+        g.attrs.create(self.LOCKED, rs.isLocked(), dtype=bool)
 
         if rs.numberOfResults() > 0:
             # ---- PART 1: write structure ---
@@ -262,7 +272,8 @@ class HDF5LabNotebook(LabNotebook):
 
             # write out each result
             df = rs.dataframe()
-            dtype = ds.dtype
+            dtype = rs.dtype()
+            hdf5dtype = ds.dtype
             dfnames = dtype.names
             for i in range(len(df.index)):
                 # get the row to write
@@ -275,10 +286,17 @@ class HDF5LabNotebook(LabNotebook):
                         # patch known datestamps ISO-format strings in metadata
                         dt = res[k].isoformat()
                         entry.append(dt)
-                    elif dtype[k] == h5py.string_dtype():
-                        entry.append(self._asString(res[k]))
                     else:
-                        entry.append(res[k])
+                        et = h5py.check_vlen_dtype(hdf5dtype[k])
+                        if et is None:
+                            # "normal" type, pass through
+                            entry.append(res[k])
+                        elif et == str:
+                            # string, convert
+                            entry.append(self._asString(res[k]))
+                        else:
+                            # array of something
+                            entry.append(numpy.array(res[k]))
 
                 # write out the row
                 ds[i] = tuple(entry)
@@ -290,11 +308,11 @@ class HDF5LabNotebook(LabNotebook):
                 # table isn't needed any more, so delete it to keep things tidy
                 del g[self.PENDINGRESULTS_DATASET]
         else:
-            pdtype = rs.pendingdtype() 
+            pdtype = rs.pendingdtype()
             hdf5pdtype = self._HDF5dtype(pdtype)
 
             # create results dataset if there isn't one
-            if self.PENDINGRESULTS_DATASET not in g:        
+            if self.PENDINGRESULTS_DATASET not in g:
                 # construct the HDF5 type of the pending results
                 # create the pending results dataset
                 g.create_dataset(self.PENDINGRESULTS_DATASET, (self.DefaultDatasetSize,), maxshape=(None,), dtype=hdf5pdtype)
@@ -336,6 +354,7 @@ class HDF5LabNotebook(LabNotebook):
         g = self._file[tag]
 
         # read the attributes, all treated as strings
+        formats = None
         locked = False
         for k in g.attrs.keys():
             if k == self.DESCRIPTION:
@@ -347,7 +366,7 @@ class HDF5LabNotebook(LabNotebook):
             else:
                 rs[k] = self._asString(g.attrs[k])
 
-        if self.RESULTS_DATASET in g:   
+        if self.RESULTS_DATASET in g:
             # ---- PART 1: read structure ---
 
             # get the HDF5 dataset associated with this result set
@@ -364,15 +383,20 @@ class HDF5LabNotebook(LabNotebook):
 
             # read each line of the dataset into the result set
             # sd: this uses the results dict API and so is quite wasteful
-            dtype = ds.dtype
+            hdf5dtype = ds.dtype
             for i in range(len(ds)):
                 entry = list(ds[i])
                 rc = Experiment.resultsdict()
                 j = 0
                 for d in [ Experiment.METADATA, Experiment.PARAMETERS, Experiment.RESULTS ]:
                     for k in names[d]:
-                        if dtype[k] == h5py.string_dtype():
+                        et = h5py.check_vlen_dtype(hdf5dtype[k])
+                        if et == str:
+                            # string, convert
                             entry[j] = self._asString(entry[j])
+                        elif et is not None:
+                            # array of something
+                            entry[j] = numpy.array(entry[j])
                         if k in [ Experiment.START_TIME, Experiment.END_TIME ]:
                             try:
                                 # patch known datestamps to datetime objects
@@ -385,7 +409,7 @@ class HDF5LabNotebook(LabNotebook):
                         j += 1
                 self.addResult(rc, tag)
 
-        if self.PENDINGRESULTS_DATASET in g:   
+        if self.PENDINGRESULTS_DATASET in g:
             # ---- PART 3: read pending results ---
 
             # get the pending results dataset
@@ -416,7 +440,7 @@ class HDF5LabNotebook(LabNotebook):
                 # record the pending result
                 self.addPendingResult(params, jobid, tag)
 
-        # lock the result set if flagged 
+        # lock the result set if flagged
         if locked:
             rs.finish()
 
@@ -432,6 +456,10 @@ class HDF5LabNotebook(LabNotebook):
                 # result set has changed since it was created, loaded, or last saved
                 self._write(tag)
 
+        # delete any datasets that aare no longer in the notebook,
+        # i.e., those that have been deleted
+        self._purge()
+
         # write out the housekeeping information for the notebook
         meta = self._file.attrs
         meta[self.CREATOR] = PackageContactInfo
@@ -480,8 +508,16 @@ class HDF5LabNotebook(LabNotebook):
         :returns: the HDF5 dtype'''
         if dtype.kind in ['S', 'U']:
             # h5py can't handle Python strings but has its own dtype for them
-            dtype = h5py.string_dtype()
-        return dtype
+            return h5py.string_dtype()
+        #elif dtype.kind in ['O']:
+        #    # object (probably numpy array), wrap as vlen
+        #    return h5py.vlen_dtype(dtype)
+        elif dtype.shape != ():
+            # list, make into a vlen type
+            return h5py.vlen_dtype(self._HDF5dtype(dtype.base))
+        else:
+            # leave unpatched
+            return dtype
 
     def _HDF5dtype(self, dtype : numpy.dtype) -> numpy.dtype:
         '''Patch a ``numpy`` dtype into its HDF5 equivalent. This method
@@ -493,7 +529,7 @@ class HDF5LabNotebook(LabNotebook):
             # a structured dtype, unpack and patch all the fields
             elements = []
             for k in dtype.names:
-                t = self._HDF5simpledtype(dtype.fields[k][0])
+                t = self._HDF5dtype(dtype.fields[k][0])
                 elements.append((k, t))
             return numpy.dtype(elements)
         else:
@@ -502,9 +538,9 @@ class HDF5LabNotebook(LabNotebook):
 
     def _asString(self, o : Any) -> str:
         '''Coerce the given value to a string. This is needed to handle
-        occasional (and Python-version-dependent) weirdness in the 
+        occasional (and Python-version-dependent) weirdness in the
         representation of strings as raw sequences when going to and from
-        HDF5. 
+        HDF5.
 
         :param o: the object
         :returns: the string'''
